
typedef unsigned int word; //Назначение псевдонима

#define OUT_SHIM 9 //Выходной порт ШИМ
#define OUT_LED 8 //Выходной порт светодиодной индикации
#define OUT_TON 7 //Выходной порт звуковой сигнализации
#define ADC_CUR 0 //Входной порт измерения тока
#define ADC_VOLT 1 //Входной порт измерения напряжения
#define IN_KEY 2 //Входной порт кнопки управления
#define REZ_FREQ 3900 //Резонансная частота пьезоэлемента

boolean Cur_Indx=false; //Индекс наличия тока
boolean Ind_Indx=false; //Индекс переключения индикации
boolean Reg_Indx=false; //Индекс изменения мощности
boolean Clb_Indx=false; //Индекс определения параметров электропаяльника
boolean Blk_Indx=false; //Индекс блокировки 
boolean Rej_Indx=false; //Индекс изменения режима
boolean Vie_Indx=false; //Индекс выхода тока за предельные значения
boolean Rvr_Indx=false; //Индекс реверса счётчика режимов
boolean Nkn_Indx=false; //Индекс нажатой кнопки
boolean Okn_Indx=true; //Индекс отжатой кнопки
byte Reg_Shim=255; //Загрузка в регистр управления мощностью минимального значения
byte Clc_Rej=2; //Установка счётчика режимов в режим 3
byte Clc_Ind=0; //Счётчик длительности индикации
byte Clc_Scn=0; //Счётчик длительности сканирования кнопки
word Power_Solder=0; //Измеренная мощность паяльника
word Step_Power=0; //Шаг изменения мощности
word Spec_Power=0; //Заданная мощность паяльника
word Inst_Power=0; //Текущая мощность паяльника
word Res_Solder=0; //Сопротивление нагревателя паяльника
word Cur_Solder=0; //Токущий ток паяльника
word Volt_Solder=0; //Текущее напряжение на паяльнике
word Clc_Time=0; //Общий счётчик времени
long Addreg_tmp=0; //Регистр хранения промежуточных результатов

void setup()
  {
  pinMode(OUT_SHIM,OUTPUT); //Определение входных и выходных портов
  pinMode(OUT_LED,OUTPUT);
  pinMode(ADC_CUR,INPUT);
  pinMode(ADC_VOLT,INPUT);
  pinMode(IN_KEY,INPUT);
  digitalWrite(IN_KEY,HIGH); //Включение подтягиющего резистора
  digitalWrite(OUT_LED,HIGH); //Включение светодиода
  analogReference(DEFAULT); //Определение источника опорного напряжения АЦП +5 В
  analogWrite(OUT_SHIM,Reg_Shim); //Загрузка регистра ШИМ
  }

void loop()
  {
  delay(1); //Выдержка времени 1 мсек
  Clc_Scn++; //Инкремент счётчика сканирования
  Clc_Time++; //Инкремент счётчика времени
  Rsm_Cur(); //Измерение действующего тока паяльника
  if((Clb_Indx==false)&&(Blk_Indx==false)) //Если параметры паяльника не определены и индекс блокировки сброшен
    {
    if((Clc_Time>100)&&(Cur_Indx==false)&&(Reg_Shim==255)) Clc_Time=0; //Подготовка счётчика времени
    else if((Clc_Time==100)&&(Cur_Indx==true)&&(Reg_Shim>1)) //Если через 100 мсек ток в норме но мощность не максимальная
      {
      if(Reg_Shim>2) Reg_Shim-=2; //Увеличение мощности
      else Reg_Shim--;
      Reg_Indx=true; //Установка индекса регулирования мощности
      Clc_Time=0; //Подготовка счётчика к следующему циклу
      }
    else if((Clc_Time>=50000)&&(Cur_Indx==true)) //Если мощность максимальна, ток в норме и прошла минута нагрева паяльника
      {
      Rsm_Volt(); //Измерение напряжения на паяльнике
      if((Volt_Solder/Cur_Solder)==Res_Solder) //Если сопротивление нагревателя паяльника равно предыдущему
        {
        Addreg_tmp=1296000/long(Res_Solder); //Определение номинальной мощности паяльника
        Power_Solder=word(Addreg_tmp); //Сохранение мощности в соответствующем регистре
        Step_Power=Power_Solder/5; //Определение шага переключения мощности
        Reg_Shim=128; //Установка средней мощности
        Reg_Indx=true; //Установка индекса регулирования мощности
        Rej_Indx=true; //Установка индекса изменения режима
        Clb_Indx=true; //Установка индекса определения параметров паяльника
        Clc_Time=0; //Подготовка счётчика
        tone(OUT_TON,REZ_FREQ,50); //Вывод звукого сигнала 50 мсек
        }
      else Res_Solder=Volt_Solder/Cur_Solder; //Если сопротивление нагревателя паяльника не равно предыдущему то сохранение последнего значения
      }
    else if((Clc_Time>100)&&(Cur_Indx==false)&&(Reg_Shim<255)) //Если ток выше или ниже предельных значений
      {
      Reg_Shim=255; //Отключение тока паяльника
      Reg_Indx=true; //Установка индекса регулирования мощности
      Ind_Indx=true; //Установка индекса переключения индикации
      Blk_Indx=true; //Установка индекса блокировки
      Clc_Time=0; //Подготовка счётчика
      }
    }
  else if((Clb_Indx==true)&&(Blk_Indx==false)) //Если параметры паяльника определены и нет блокировки
    {
    if((digitalRead(IN_KEY)==HIGH)&&(Nkn_Indx==false)&&(Okn_Indx==false)) Okn_Indx=true; //Если кнопка отжата а индек установлен то сброс индекса отжатой кнопки
    else if((digitalRead(IN_KEY)==LOW)&&(Nkn_Indx==false)&&(Okn_Indx==true)) //Если обнаружена нажатая кнопка 
      {
      Nkn_Indx=true; //Установка индекса нажатой кнопки
      Okn_Indx=false; //Сброс индекса отжатой кнопки
      Clc_Scn=0; //Подготовка счётчика сканирования
      }
    else if((digitalRead(IN_KEY)==HIGH)&&(Nkn_Indx==true)&&(Okn_Indx==false)&&(Clc_Scn>=20)) //Если через 20 мсек нажатая кнопка не обнаружена
      {
      Nkn_Indx=false; //Сброс индекса нажатой кнопки
      Okn_Indx=true; //Установка индекса отжатой кнопки
      }
    else if((digitalRead(IN_KEY)==LOW)&&(Nkn_Indx==true)&&(Okn_Indx==false)&&(Clc_Scn>=20)) //Если через 20 мсек нажатая кнопка обнаружена
      {
      Nkn_Indx=false; //Сброс индекса нажатой кнопки
      Rej_Indx=true; //Установка индекса изменения режима
      if((Rvr_Indx==false)&&(Clc_Rej<4)) //Если индекс реверса сброшен то прямой счёт счётчика режимов
        {
        Clc_Rej++; //Инкремент счётчика режимов
        if(Clc_Rej==4) Rvr_Indx=true; //Если счётчик равен максимальному значению четыре то установка индекса реверса
        }
      else if((Rvr_Indx==true)&&(Clc_Rej>0)) //Если индекс реверса установлен то обратный счёт счётчика режимов
        {
        Clc_Rej--; //Декремент счётчика режимов
        if(Clc_Rej==0) Rvr_Indx=false; //Если счётчик равен нулю то сброс индекса реверса
        }
      tone(OUT_TON,REZ_FREQ,20); //Вывод звукового сигнала нажатой кнопки 20 мсек
      }
    if(Clc_Time>=200) //Выдержка 200 мсек
      {
      Clc_Time=0; //Подготовка счётчика
      if(Cur_Indx==true) //Если ток в норме
        {
        Pow_Current(); //Определение теущей потребляемой мощности паяльником
        if((Inst_Power<Spec_Power)&&(Reg_Shim>1)) //Если потребляемая мощность меньше заданной
          {
          Reg_Shim--; //Декремент регистра управления мощностью
          Reg_Indx=true; //Установка индекса регулирования мощности
          }
        else if((Inst_Power>Spec_Power)&&(Reg_Shim<255)) //Если потребляемая мощность больше заданной
          {
          Reg_Shim++; //Инкремент регистра управления мощностью
          Reg_Indx=true; //Установка индекса регулирования мощности
          }
        if(Ind_Indx==false) //Если индекс индикации сброшен
          {
          Ind_Indx=true; //Установка индекса
          if(Clc_Rej>=Clc_Ind) digitalWrite(OUT_LED,HIGH); //Если содержимо счётчика режимов больше или равно содержимому счётчика длительности индикации светодиод включен
          ++Clc_Ind; //Инкремент счётчика длительности индикации
          if(Clc_Ind>=10) Clc_Ind=0; //Если содержимое счётчика больше или равно 10 то его сброс
          }
        else //Если индекс индикации установлен
          {
          Ind_Indx=false; //Сброс индекса
          digitalWrite(OUT_LED,LOW); //Светодиод отключен
          }
        }
      else //Если ток за пределами нормы
        {
        Reg_Shim=255; //Загрузка регистра управления мощностью для отключения тока
        Reg_Indx=true; //Установка индекса регулирования мощности
        Ind_Indx=true; //Установка индекса индикации
        Blk_Indx=true; //Установка индекса блокировки
        Clc_Ind=0; //Сброс счётчика длительности индикации
        }
      }
    }
  else if(Blk_Indx==true) //Если индекс блокировки установлен
    {
    if(Clc_Time>=100) //Выдержка времени 100 мсек
      {
      Clc_Time=0; //Подготовка счётчика
      if(Vie_Indx==false) //Если ток ниже нормы
        {
        if((((Clc_Ind>=0)&&(Clc_Ind<=2))||((Clc_Ind>=25)&&(Clc_Ind<=27)))&&(Ind_Indx==true)) //При совпадении условий содержимого счётчика длительности идикации и индекс индикации установлен
          {
          Ind_Indx=false; //Сброс индекса
          digitalWrite(OUT_LED,HIGH); //Включение светодиода
          }
        else if(Ind_Indx==false) //Если индекс индикации сброшен
          {
          Ind_Indx=true; //Установка индекса
          digitalWrite(OUT_LED,LOW); //Отключение светодиода
          }
        }
      else if(Vie_Indx==true) //Если ток выше нормы
        {
        if((((Clc_Ind>=0)&&(Clc_Ind<=2))||((Clc_Ind>=25)&&(Clc_Ind<=27)))&&(Ind_Indx==true)) //При совпадении условий содержимого счётчика длительности идикации и индекс индикации установлен
          {
          Ind_Indx=false; //Сброс индекса
          digitalWrite(OUT_LED,LOW); //Отключение светодиода
          }
        else if(Ind_Indx==false) //Если индекс индикации сброшен
          {
          Ind_Indx=true; //Установка индекса
          digitalWrite(OUT_LED,HIGH); //Включение светодиода
          }
        }
      tone(OUT_TON,REZ_FREQ,100); //Вывод звукового сигнала 100 мсек
      ++Clc_Ind; //Инкремент счётчика длительности индикации
      if(Clc_Ind>=50) //Если содержимое счётчика 50 и более
        {
        if((Cur_Indx==true)&&(Vie_Indx==false)) //Если ток в норме
          {
          digitalWrite(OUT_LED,HIGH); //Включение светодиода
          Ind_Indx=false; //Сброс индекса индикации
          Blk_Indx=false; //Сброс индекса блокировки
          }
        Clc_Ind=0; //Сброс счётчика длительности индикации
        }
      }
    }
  if(Rej_Indx==true) //Если индекс изменения режима установлен
    {
    Rej_Indx=false; //Сброс индекса изменения режима
    Spec_Power=Step_Power*(word(Clc_Rej)+1); //Определение нового значения заданной мощности
    }
  if(Reg_Indx==true) //Если установлен индекс регулирования мощности
    {
    Reg_Indx=false; //Сброс индекса
    analogWrite(OUT_SHIM,Reg_Shim); //Установка нового режима ШИМ
    }
  }

void Rsm_Cur(void) //Функция определения действующего значения тока паяльника
  {
  long voltres_tmp=0; //Определение переменных
  long rsmcur_tmp=0;
  long coefzpl_tmp=0;
  coefzpl_tmp=sqrt(2560000/(256-Reg_Shim)); //Определение коэффициента заполнения
  voltres_tmp=long(analogRead(ADC_CUR))*coefzpl_tmp/20; //Измерение и рсчёт значения напряжения на токоизмерительном резисторе
  rsmcur_tmp=voltres_tmp*100/204; //Определение тока паяльника
  Cur_Solder=word(rsmcur_tmp); //Загрузка значения тока в регистр
  if((Reg_Shim<240)&&(Cur_Solder<20)) //Если ток ниже нормы
    {
    Vie_Indx=false; //Сброс индекса выхода тока за пределы
    Cur_Indx=false; //Сброс индекса наличия тока
    }
  else if(Cur_Solder>1200) //Если ток выше нормы
    {
    Vie_Indx=true; //Установка индекса выхода тока за пределы
    Cur_Indx=false; //Сброс индекса наличия тока
    }
  else Cur_Indx=true; //Установка индекса если ток в норме
  }

void Rsm_Volt(void) //Функция измерения входного напряжения
  {
  long voltin_tmp=0; //Определение переменных
  long voltres_tmp=0;
  voltin_tmp=long(analogRead(ADC_VOLT))*89; //Измерение входного напряжения
  voltres_tmp=analogRead(ADC_CUR)*5; //Измерение напряжения на токоизмерительном резисторе
  voltin_tmp-=voltres_tmp; //Коррекция входного напряжения
  Volt_Solder=word(voltin_tmp); //Загрузка значения входного напряжения в регистр
  }

void Pow_Current(void) //Функция определения текущей мощности потребляемой паяльником
  {
  long powsold_tmp=0; //Определение переменной
  powsold_tmp=sq(long(Cur_Solder))*long(Res_Solder)/1000; //Расчёт потребляемой мощности
  Inst_Power=word(powsold_tmp); //Загрузка значения текущей мощности в регистр
  }

